
import unittest

from webtest import TestApp


class BrowserTestCase(unittest.TestCase):

    layer = None

    def setUp(self):
        super(BrowserTestCase, self).setUp()
        self._testapp = TestApp(self.layer.make_wsgi_app())

    def publish(self, path, basic=None, headers=None, handle_errors=False):
        assert basic
        self._testapp.authorization = ('Basic', tuple(basic.split(':')))
        env = {'wsgi.handleErrors': handle_errors}
        response = self._testapp.get(path, extra_environ=env, headers=headers,
                                     expect_errors=handle_errors)
        return response



##############################################################################
#
# Copyright (c) 2001-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Unauthorized Exception Views"""

import unittest
from zope import component, interface
import zope.browserpage.namedtemplate
from zope.publisher.browser import TestRequest
from zope.authentication.interfaces import IAuthentication
from zope.security.interfaces import IPrincipal

from zope.app.exception.browser.unauthorized import Unauthorized
from zope.component.testing import PlacelessSetup


@interface.implementer(IPrincipal)  # this is a lie
class DummyPrincipal(object):

    def __init__(self, id):
        self.id = id

    def getId(self):  # pragma: no cover
        return self.id


@interface.implementer(IAuthentication)  # this is a lie
class DummyAuthUtility(object):

    status = None

    def unauthorized(self, principal_id, request):
        self.principal_id = principal_id
        self.request = request
        if self.status is not None:
            self.request.response.setStatus(self.status)


@component.adapter(Unauthorized)
@interface.implementer(zope.browserpage.namedtemplate.INamedTemplate)
class DummyTemplate(object):

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return 'You are not authorized'


class Test(PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super(Test, self).setUp()
        self.auth = DummyAuthUtility()
        component.provideUtility(self.auth, IAuthentication)

    def tearDown(self):
        super(Test, self).tearDown()

    def testUnauthorized(self):
        component.provideAdapter(DummyTemplate, name="default")
        exception = Exception()
        try:
            raise exception
        except:  # noqa: E722 do not use bare 'except'
            pass
        request = TestRequest()
        request.setPrincipal(DummyPrincipal(23))
        u = Unauthorized(exception, request)
        res = u()

        # Make sure that we rendered the expected template
        self.assertEqual("You are not authorized", res)

        # Make sure the response status was set
        self.assertEqual(request.response.getStatus(), 403)

        # check headers that work around squid "negative_ttl"
        self.assertEqual(request.response.getHeader('Expires'),
                         'Mon, 26 Jul 1997 05:00:00 GMT')
        self.assertEqual(request.response.getHeader('Pragma'),
                         'no-cache')
        self.assertEqual(request.response.getHeader('Cache-Control'),
                         'no-store, no-cache, must-revalidate')

        # Make sure the auth utility was called
        self.assertIs(self.auth.request, request)
        self.assertEqual(self.auth.principal_id, 23)

    def testRedirect(self):
        exception = Exception()
        try:
            raise exception
        except:  # noqa: E722 do not use bare 'except'
            pass
        request = TestRequest()
        request.setPrincipal(DummyPrincipal(23))
        u = Unauthorized(exception, request)

        self.auth.status = 303

        res = u()

        # Make sure that the template was not rendered
        self.assertIsNone(res)

        # Make sure the auth's redirect is honored
        self.assertEqual(request.response.getStatus(), 303)

        # Make sure the auth utility was called
        self.assertIs(self.auth.request, request)
        self.assertEqual(self.auth.principal_id, 23)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""NotFound Error View class.
"""
__docformat__ = 'restructuredtext'


class NotFound(object):
    """`NotFound` Error View

    `NotFound` errors should return 404 instead of 200.
    """

    def __call__(self, *args, **kw):
        self.request.response.setStatus(404)
        return self.index(*args, **kw)



##############################################################################
#
# Copyright (c) 2006-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.app.exception package
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


version = '4.1.0'

tests_require = [
    'webtest',

    'zope.app.appsetup',
    'zope.app.basicskin >= 4.0',
    'zope.app.pagetemplate',
    'zope.app.publication',
    'zope.app.wsgi',

    'zope.browsermenu',
    'zope.browserresource',
    'zope.container',
    'zope.login',
    'zope.password',
    'zope.principalregistry',
    'zope.publisher',
    'zope.securitypolicy',
    'zope.site',
    'zope.testrunner',
]

setup(name='zope.app.exception',
      version=version,
      author='Zope Corporation and Contributors',
      author_email='zope-dev@zope.org',
      description='Zope 3 exception views',
      long_description=(
          read('README.rst')
          + '\n\n' +
          '.. contents::'
          + '\n\n' +
          read('src', 'zope', 'app', 'exception', 'browser', 'systemerror.rst')
          + '\n\n' +
          read('CHANGES.rst')
      ),
      keywords="zope3 exception view",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope :: 3',
      ],
      url='http://pypi.python.org/pypi/zope.app.exception',
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['zope', 'zope.app'],
      extras_require={
          'test': tests_require,
      },
      tests_require=tests_require,
      install_requires=[
          'setuptools',
          'zope.interface',
          'zope.publisher >= 3.12',
          'zope.authentication',
          'zope.browser>=1.2',
          'zope.browserpage>=3.11.0',
          'zope.component',
          'zope.security',
      ],
      include_package_data=True,
      zip_safe=False,
      )



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Functional tests for NotFoundError
"""
import doctest
import unittest
from zope.app.exception.browser.tests import BrowserTestCase
from zope.app.exception.testing import AppExceptionLayer
from zope.app.wsgi.testlayer import http
from zope.interface.interfaces import ComponentLookupError


class RaiseError(object):

    def __call__(self):
        raise Exception()


class RaiseComponentLookupError(object):

    def __call__(self):
        raise ComponentLookupError()


class TestComponentLookupError(BrowserTestCase):

    layer = AppExceptionLayer

    def testComponentLookupError(self):
        response = self.publish('/foobar', basic='mgr:mgrpw',
                                handle_errors=True)
        self.assertEqual(response.status_int, 404)
        body = response.unicode_normal_body
        self.assertIn(
            'The page that you are trying to access is not available', body)


class TestMisc(unittest.TestCase):

    def test_user(self):
        from zope.app.exception.browser import user
        view = user.UserErrorView()
        view.context = self
        self.assertEqual(view.title(), self.__class__.__name__)

    def test_interfaces(self):
        from zope.app.exception import interfaces
        from zope.browser.interfaces import ISystemErrorView
        self.assertEqual(interfaces.ISystemErrorView, ISystemErrorView)


class TestUserpt(unittest.TestCase):

    layer = AppExceptionLayer

    def _render_with_context(self, context):
        from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
        from zope.publisher.browser import TestRequest
        import os
        path = os.path.join(os.path.abspath(
            os.path.dirname(__file__)), '..', 'user.pt')

        template = ViewPageTemplateFile(path)

        class Instance(object):
            def __init__(self):
                self.context = context
                self.request = TestRequest()
                self.request.setPrincipal(self)

            title = 'principal'

        instance = Instance()

        return template(instance)

    def test_render_with_exception(self):
        s = self._render_with_context(Exception("This is the message"))
        self.assertIn("This is the message", s)

    def test_render_with_iterable(self):
        s = self._render_with_context(["Just an iterable"])
        self.assertIn("Just an iterable", s)


def test_suite():
    TestComponentLookupError.layer = AppExceptionLayer

    def _http(query_str, *args, **kwargs):
        wsgi_app = AppExceptionLayer.make_wsgi_app()
        # Strip leading \n
        query_str = query_str.lstrip()
        kwargs.setdefault('handle_errors', True)
        if not isinstance(query_str, bytes):  # always true on PY3
            query_str = query_str.encode("utf-8")
        return http(wsgi_app, query_str, *args, **kwargs)

    systemerror = doctest.DocFileSuite(
        '../systemerror.rst',
        optionflags=doctest.ELLIPSIS,
        globs={'http': _http})

    systemerror.layer = AppExceptionLayer
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        systemerror,
    ))
